@@ -2,19 +2,12 @@ import math
 import os
 from pgmagick import *
 
-# Draw extra lines showing where the margins and text centers are
-DEBUG = False
-
-# Used for navigating the array of data to plot
-DATA_LABEL_INDEX = 0
-DATA_VALUES_INDEX = 1
-DATA_COLOR_INDEX = 2
-
 # Space around text elements. Mostly used for vertical spacing
 LEGEND_MARGIN = TITLE_MARGIN = 20.0
 LABEL_MARGIN = 10.0
 DEFAULT_MARGIN = 20.0
 
+DEFAULT_TRANSPARENCY = 0.7
 DEFAULT_TARGET_WIDTH = 800
 DEFAULT_FONT = "Vera.ttf"
 
@@ -22,6 +15,25 @@ THOUSAND_SEPARATOR = ','
 
 
 class Base(object):
+    """This object is based on all Graph Object.
+    When you create new graph object, override this object::
+
+        from gruffy.base import Base
+
+        class NewGraph(Base):
+            pass
+    """
+
+    #: Title of Graph.
+    title = None
+
+    #: Transparent flag (or value).
+    #: *False* is non-transparent rendering, *True* is default value
+    #: transparent, *float* value is value's transparent.
+    transparent = False
+
+    #: setting labels
+    x_axis_label = y_axis_label = None
 
     def __init__(self, target_width=DEFAULT_TARGET_WIDTH):
         if type(target_width) is not int:
@@ -32,11 +44,11 @@ class Base(object):
             self.columns = float(target_width)
             self.rows = target_width * 0.75
 
-        self.initialize_ivars()
-        self.reset_themes()
-        self.theme_pastel()
+        self._initialize_ivars()
+        self._reset_themes()
+        self.theme_keynote()
 
-    def initialize_ivars(self):
+    def _initialize_ivars(self):
         """Internal for calculations"""
         self.raw_columns = 800.0
         self.raw_rows = 800.0 * (self.rows / self.columns)
@@ -48,13 +60,12 @@ class Base(object):
         self.labels = {}
         self.labels_seen = {}
         self.sort = True
-        self.title = None
 
         self.scale = self.columns / self.raw_columns
 
         if 'MAGICK_FONT_PATH' in os.environ:
             vera_font_path = os.path.join(os.environ['MAGICK_FONT_PATH'],
-                                          'Vera.ttf')
+                                          DEFAULT_FONT)
         else:
             vera_font_path = 'Vera.ttf'
         self.font = vera_font_path if os.path.exists(vera_font_path) else None
@@ -85,21 +96,18 @@ class Base(object):
         self.additional_line_colors = []
         self.theme_options = {}
 
-        self.x_axis_label = self.y_axis_label = None
         self.y_axis_increment = None
         self.stacked = None
         self.norm_data = None
 
-    def reset_themes(self):
+    def _reset_themes(self):
         self.color_index = 0
         self.labels_seen = {}
         self.theme_options = {}
-        #self.d = Image()
-        # Scale down from 800x600 used to calculate drawing.
-        #self.d.transformScale(self.scale, self.scale)
 
     def theme_keynote(self):
-        # Colors
+        """Setting up Keynote like gradient theme.
+        """
         self.blue = '#6886B4'
         self.yellow = '#FDD84E'
         self.green = '#72AE6E'
@@ -110,14 +118,65 @@ class Base(object):
         self.colors = [self.yellow, self.blue, self.green, self.red,
                        self.purple, self.orange, self.white]
 
-        self.theme({
-                'colors': self.colors,
-                'marker_color': 'white',
-                'font_color': 'white',
-                'background_colors': ['black', '#4a465a'],
-        })
+        self.set_theme({'colors': self.colors,
+                        'marker_color': 'white',
+                        'font_color': 'white',
+                        'background_colors': ['black', '#4a465a']})
+
+    def theme_37signals(self):
+        """Setting up 37 signals like gradient theme.
+        """
+        self.green = '#339933'
+        self.purple = '#cc99cc'
+        self.blue = '#336699'
+        self.yellow = '#FFF804'
+        self.red = '#ff0000'
+        self.orange = '#cf5910'
+        self.black = 'black'
+        self.colors = [self.yellow, self.blue, self.green, self.red,
+                       self.purple, self.orange, self.black]
+        self.set_theme({'colors': self.colors,
+                        'marker_color': 'black',
+                        'font_color': 'black',
+                        'background_colors': ['#d1edf5', 'white']})
+
+    def theme_rails_keynote(self):
+        """A color scheme from the colors used on the 2005 Rails keynote
+        presentation at RubyConf.
+        """
+        self.green = '#00ff00'
+        self.grey = '#333333'
+        self.orange = '#ff5d00'
+        self.red = '#f61100'
+        self.white = 'white'
+        self.light_grey = '#999999'
+        self.black = 'black'
+        self.colors = [self.green, self.grey, self.orange, self.red,
+                       self.white, self.light_grey, self.black]
+        self.set_theme({'colors': self.colors,
+                        'marker_color': 'white',
+                        'font_color': 'white',
+                        'background_colors': ['#0083a3', '#0083a3']})
+
+    def theme_odeo(self):
+        """A color scheme similar to that used on the popular podcast site.
+        """
+        self.grey = '#202020'
+        self.white = 'white'
+        self.dark_pink = '#a21764'
+        self.green = '#8ab438'
+        self.light_grey = '#999999'
+        self.dark_blue = '#3a5b87'
+        self.black = 'black'
+        self.colors = [self.grey, self.white, self.dark_blue,
+                       self.dark_pink, self.green, self.light_grey, self.black]
+        self.set_theme({'colors': self.colors,
+                        'marker_color': 'white',
+                        'font_color': 'white',
+                        'background_colors': ['#ff47a4', '#ff1f81']})
 
     def theme_pastel(self):
+        """Setting up pastel color theme."""
         self.colors = ['#a9dada',   # blue
                        '#aedaa9',   # green
                        '#daaea9',   # peach
@@ -126,38 +185,59 @@ class Base(object):
                        '#daaeda',   # purple
                        '#dadada',   # grey
                       ]
+        self.set_theme({'colors': self.colors,
+                        'marker_color': '#aea9a9',
+                        'font_color': 'black',
+                        'background_colors': 'white'})
 
-        self.theme({'colors': self.colors,
-                    'marker_color': '#aea9a9',  # Grey
-                    'font_color': 'black',
-                    'background_colors': 'white'})
+    def theme_django(self):
+        """Setting up Django like theme"""
+        self.colors = ['#a9dada', '#aedaa9', '#94da3a',
+                       '#487854', '#a9a9da', '#dadada']
+        self.set_theme({'colors': self.colors,
+                        'marker_color': '#ab5603',
+                        'font_color': 'white',
+                        'background_colors': ['#092e20', '#234f32']})
 
     def theme_greyscale(self):
+        """Setting up black and white theme"""
         self.colors = ['#282828', '#383838', '#686868',
                        '#989898', '#c8c8c8', '#e8e8e8']
-        self.theme({'colors': self.colors,
-                    'marker_color': '#aea9a9',
-                    'font_color': 'black',
-                    'background_colors': 'white'})
+        self.set_theme({'colors': self.colors,
+                        'marker_color': '#aea9a9',
+                        'font_color': 'black',
+                        'background_colors': 'white'})
 
-    def render_gradiated_background(self, top_color, bottom_color):
-        # TODO: porting GradientFill
-        return Image(Geometry(int(self.columns), int(self.rows)),
-                     Color('gradient:black-white'))
+    def render_background(self):
+        if type(self.theme_options['background_colors']) is list:
+            colors = self.theme_options['background_colors']
+            self.base_image = self.render_gradiated_background(*colors)
+        elif type(self.theme_options['background_colors']) is str:
+            colors = self.theme_options['background_colors']
+            self.base_image = self.render_solid_background(colors)
+        else:
+            image = self.theme_options['background_image']
+            self.base_image = self.render_image_background(image)
 
     def render_solid_background(self, color):
         return Image(Geometry(int(self.columns), int(self.rows)), color)
 
-    def render_background(self):
-        if type(self.theme_options['background_colors']) is list:
-            self.base_image = self.render_gradiated_background(*self.theme_options['background_colors'])
-        elif type(self.theme_options['background_colors']) is str:
-            self.base_image = self.render_solid_background(self.theme_options['background_colors'])
-        else:
-            self.base_image = self.render_image_background(*self.theme_options['background_image'])
+    def render_gradiated_background(self, top_color, bottom_color):
+        im = Image(Geometry(int(self.columns), int(self.rows)),
+                   Color('transparent'))
+        im.read("gradient:%s-%s" % (top_color, bottom_color))
+        return im
 
-    def theme(self, options):
-        self.reset_themes()
+    def render_image_background(self, image_path, opacity=True):
+        image = Image(image_path)
+        if type(opacity) is int:
+            image.opacity(opacity)
+        elif opacity is True:
+            image.opacity(50)
+        return image
+
+    def set_theme(self, options):
+        self._reset_themes()
 
         defaults = {
                 'colors': ['black', 'white'],
@@ -180,16 +260,24 @@ class Base(object):
         return self.colors[self.color_index - 1]
 
     def data(self, name, data_points=[], color=None):
+        """Set up graph dataset
+
+        :param name: data name
+        :param data_points: list of data point
+        :param color: *force* settig color of graph data
+        """
         data_points = list(data_points)
-        self.gdata.append([name, data_points, (color or self.increment_color())])
-        self.column_count = len(data_points) if len(data_points) > self.column_count else self.column_count
+        self.gdata.append({'label': name,
+                           'values': data_points,
+                           'color': color or self.increment_color()})
+        if len(data_points) > self.column_count:
+            self.column_count = len(data_points)
         for cnt, data_point in enumerate(data_points):
             if data_points is None:
                 continue
             if self.maximum_value is None and self.minimum_value is None:
                 self.maximum_value = self.minimum_value = data_point
             # TODO Doesn't work with stacked bar graphs
-            # Original:@maximum_value = larger_than_max?(data_point, index) ? max(data_point, index) : @maximum_value
             if self.larger_than_max(data_point):
                 self.maximum_value = data_point
             if self.maximum_value >= 0:
@@ -199,11 +287,6 @@ class Base(object):
             if self.minimum_value < 0:
                 self.has_gdata = True
 
-    def annotate_scaled(self, width, height, text, scale, gravity):
-        scaled_width = (width * scale) if (width * scale) >= 1 else 1
-        scaled_height = (height * scale) if (height * scale) >= 1 else 1
-        self.base_image.annotate(text, gravity)
-
     def draw_line_markers(self):
         if self.hide_line_markers:
             return
@@ -219,10 +302,14 @@ class Base(object):
                         self.marker_count = lines
                         break
                 self.marker_count = self.marker_count or 4
-            self.increment = self.significant(self.spread / self.marker_count) if (self.spread > 0) else 1
+            if self.spread > 0:
+                self.increment = self.significant(self.spread / self.marker_count)
+            else:
+                self.increment = 1
         else:
             # TODO Make this work for negative values
-            self.maximum_value = [self.maximum_value.ceil, self.y_axis_increment].max
+            self.maximum_value = max([self.maximum_value.ceil,
+                                      self.y_axis_increment])
             self.minimum_value = math.floor(self.minimum_value)
             self.calculate_spread()
             self.normalize(true)
@@ -233,7 +320,7 @@ class Base(object):
 
         # Draw horizontal line markers and annotate with numbers
         dl = DrawableList()
-        for index in range(self.marker_count+1):
+        for index in range(self.marker_count + 1):
             y = self.graph_top + self.graph_height - float(index) * self.increment_scaled
 
             dl.append(DrawableFillColor(Color(self.marker_color)))
@@ -247,45 +334,23 @@ class Base(object):
                 dl.append(DrawableFont(font, StyleType.NormalStyle, 400,
                                        StretchType.NormalStretch))
                 dl.append(DrawableStrokeColor('transparent'))
-                dl.append(DrawablePointSize(self.scale_fontsize(self.marker_font_size)))
+                marker_font_size = self.scale_fontsize(self.marker_font_size)
+                dl.append(DrawablePointSize(marker_font_size))
 
                 # Vertically center with 1.0 for the height
                 dl.append(DrawableGravity(GravityType.NorthWestGravity))
                 x = self.calculate_width(self.marker_font_size, marker_label) / 2
-                y = y + self.calculate_caps_height(self.scale_fontsize(self.marker_font_size)) / 3
+                y = y + self.calculate_caps_height(marker_font_size) / 3
                 dl.append(DrawableText(self.graph_left - LABEL_MARGIN - x,
                                        y, self.label(marker_label)))
-
-        # # Submitted by a contibutor...the utility escapes me
-        # i = 0
-        # self.additional_line_values.each do |value|
-        #   self.increment_scaled = self.graph_height.to_f / (self.maximum_value.to_f / value)
-        #
-        #   y = self.graph_top + self.graph_height - self.increment_scaled
-        #
-        #   self.d = self.d.stroke(self.additional_line_colors[i])
-        #   self.d = self.d.line(self.graph_left, y, self.graph_right, y)
-        #
-        #
-        #   self.d.fill = self.additional_line_colors[i]
-        #   self.d.font = self.font if self.font
-        #   self.d.stroke('transparent')
-        #   self.d.pointsize = scale_fontsize(self.marker_font_size)
-        #   self.d.gravity = EastGravity
-        #   self.d = self.d.annotate_scaled( self.base_image,
-        #                     100, 20,
-        #                     -10, y - (self.marker_font_size/2.0),
-        #                     "", self.scale)
-        #   i += 1
-        # end
-
+        dl.append(DrawableScaling(self.scale, self.scale))
         dl.append(DrawableStrokeAntialias(True))
         self.base_image.draw(dl)
 
     def draw_label(self, x_offset, index):
         if self.hide_line_markers:
             return
-        if self.labels.has_key(index) and not self.labels_seen.has_key(index):
+        if index in self.labels and index not in self.labels_seen:
             y_offset = self.graph_bottom + LABEL_MARGIN
 
             dl = DrawableList()
@@ -295,13 +360,14 @@ class Base(object):
             dl.append(DrawableFont(font, StyleType.NormalStyle, 400,
                                    StretchType.NormalStretch))
             dl.append(DrawableStrokeColor(Color('transparent')))
-            dl.append(DrawablePointSize(self.scale_fontsize(self.marker_font_size)))
+            dl.append(DrawablePointSize(self.marker_font_size))
             label_font_height = self.calculate_caps_height(self.marker_font_size) / 2
             label_text_width = self.calculate_width(self.marker_font_size,
                                                     self.labels[index])
             dl.append(DrawableText(x_offset - label_text_width / 2.0,
                                    y_offset + label_font_height,
                                    self.labels[index]))
+            dl.append(DrawableScaling(self.scale, self.scale))
             self.base_image.draw(dl)
             self.labels_seen[index] = 1
 
@@ -314,8 +380,8 @@ class Base(object):
         dl.append(DrawableFont(font, StyleType.NormalStyle, 800,
                                StretchType.NormalStretch))
         dl.append(DrawablePointSize(self.scale_fontsize(80)))
-        dl.append(DrawableText(0, 0,
-                               self.no_data_message))
+        dl.append(DrawableText(0, 0, self.no_data_message))
+        dl.append(DrawableScaling(self.scale, self.scale))
         self.base_image.draw(dl)
 
     def scale_fontsize(self, value):
@@ -348,8 +414,8 @@ class Base(object):
 
     def sort_norm_data(self):
         def normcompare(a, b):
-            a_sum = sum(a[DATA_VALUES_INDEX])
-            b_sum = sum(b[DATA_VALUES_INDEX])
+            a_sum = sum(a['values'])
+            b_sum = sum(b['values'])
             if a_sum > b_sum:
                 return 1
             elif a_sum < b_sum:
@@ -358,8 +424,8 @@ class Base(object):
         self.norm_data.sort(normcompare)
         self.norm_data.reverse()
 
-    def center(self, lsize):
-        return self.raw_columns / 2 - lsize / 2
+    def center(self, size):
+        return (self.raw_columns - size) / 2
 
     def draw_axis_labels(self):
         if self.x_axis_label:
@@ -370,16 +436,20 @@ class Base(object):
             x_axis_label_y_coordinate = self.graph_bottom + \
                     LABEL_MARGIN * 2 + self.marker_caps_height
 
-            # TODO Center between graph area
             dl.append(DrawableFillColor(Color(self.font_color)))
             font = self.font if self.font else DEFAULT_FONT
             dl.append(DrawableFont(font, StyleType.NormalStyle, 400,
                                    StretchType.NormalStretch))
             dl.append(DrawableStrokeColor('transparent'))
-            dl.append(DrawablePointSize(self.scale_fontsize(self.marker_font_size)))
+            dl.append(DrawablePointSize(self.marker_font_size))
             dl.append(DrawableGravity(GravityType.NorthGravity))
+            # graph center
+            #dl.append(DrawableText(self.graph_left / 2.0,
+            #                       x_axis_label_y_coordinate,
+            #                       self.x_axis_label))
             dl.append(DrawableText(0.0, x_axis_label_y_coordinate,
                                    self.x_axis_label))
+            dl.append(DrawableScaling(self.scale, self.scale))
             self.base_image.draw(dl)
         if self.y_axis_label:
             # Y Axis, rotated vertically
@@ -389,29 +459,29 @@ class Base(object):
             dl.append(DrawableFont(font, StyleType.NormalStyle, 400,
                                    StretchType.NormalStretch))
             dl.append(DrawableStrokeColor('transparent'))
-            fontsize = self.scale_fontsize(self.marker_font_size)
+            fontsize = self.marker_font_size
             dl.append(DrawablePointSize(fontsize))
             dl.append(DrawableRotation(90))
             dl.append(DrawableGravity(GravityType.WestGravity))
             x = -(self.calculate_width(fontsize, self.y_axis_label) / 2.0)
             y = -(self.left_margin + self.marker_caps_height / 2.0)
-            print x, y
             dl.append(DrawableText(x, y, self.y_axis_label))
+            dl.append(DrawableScaling(self.scale, self.scale))
             dl.append(DrawableRotation(-90))
             self.base_image.draw(dl)
 
     def draw_legend(self):
         if self.hide_legend:
             return
-        self.legend_labels = [gdata[DATA_LABEL_INDEX] for gdata in self.gdata]
+        self.legend_labels = [gdata['label'] for gdata in self.gdata]
         legend_square_width = self.legend_box_size
 
         dl = DrawableList()
 
         font = self.font if self.font else DEFAULT_FONT
-        dl.append(DrawablePointSize(self.scale_fontsize(self.legend_font_size)))
+        dl.append(DrawablePointSize(self.legend_font_size))
 
-        label_widths = [[]] # Used to calculate line wrap
+        label_widths = [[]]     # Used to calculate line wrap
         for label in self.legend_labels:
             metrics = TypeMetric()
             self.base_image.fontTypeMetrics(str(label), metrics)
@@ -429,7 +499,7 @@ class Base(object):
         dl.append(DrawableStrokeColor('transparent'))
         for index, legend_label in enumerate(self.legend_labels):
             # Now draw box with color of this dataset
-            dl.append(DrawableFillColor(Color(self.gdata[index][DATA_COLOR_INDEX])))
+            dl.append(DrawableFillColor(Color(self.gdata[index]['color'])))
             dl.append(DrawableRectangle(current_x_offset,
                                         current_y_offset - legend_square_width / 2.0,
                                         current_x_offset + legend_square_width,
@@ -440,13 +510,13 @@ class Base(object):
             font = self.font if self.font else DEFAULT_FONT
             dl.append(DrawableFont(font, StyleType.NormalStyle, 400,
                                    StretchType.NormalStretch))
-            dl.append(DrawablePointSize(self.scale_fontsize(self.legend_font_size)))
+            dl.append(DrawablePointSize(self.legend_font_size))
             dl.append(DrawableGravity(GravityType.NorthWestGravity))
-            dl.append(DrawableText(current_x_offset + (legend_square_width * 1.7),
-                                   current_y_offset + self.legend_caps_height / 3,
-                                   str(legend_label)))
+            x = current_x_offset + legend_square_width * 1.7
+            y = current_y_offset + self.legend_caps_height / 3
+            dl.append(DrawableText(x, y, str(legend_label)))
 
-            dl.append(DrawablePointSize(self.scale_fontsize(self.legend_font_size)))
+            dl.append(DrawablePointSize(self.legend_font_size))
             metrics = TypeMetric()
             self.base_image.fontTypeMetrics(str(legend_label), metrics)
             current_string_offset = metrics.textWidth() + legend_square_width * 2.7
@@ -466,6 +536,7 @@ class Base(object):
             else:
                 current_x_offset += current_string_offset
         if len(dl):
+            dl.append(DrawableScaling(self.scale, self.scale))
             self.base_image.draw(dl)
         self.color_index = 0
 
@@ -478,9 +549,10 @@ class Base(object):
         dl.append(DrawableGravity(GravityType.NorthGravity))
         dl.append(DrawableFont(font, StyleType.NormalStyle, 800,
                                StretchType.NormalStretch))
-        dl.append(DrawablePointSize(self.scale_fontsize(self.title_font_size)))
+        dl.append(DrawablePointSize(self.title_font_size))
         y = self.top_margin + self.title_caps_height / 2.0
         dl.append(DrawableText(0, y, self.title))
+        dl.append(DrawableScaling(self.scale, self.scale))
         self.base_image.draw(dl)
 
     def setup_drawing(self):
@@ -507,15 +579,15 @@ class Base(object):
 
             for data_row in self.gdata:
                 norm_data_points = []
-                for data_point in data_row[DATA_VALUES_INDEX]:
+                for data_point in data_row['values']:
                     if data_point is None:
                         norm_data_points.append(None)
                     else:
                         norm_data_points.append(((float(data_point) - \
                                 float(self.minimum_value)) / self.spread))
-                self.norm_data.append([data_row[DATA_LABEL_INDEX],
-                                       norm_data_points,
-                                       data_row[DATA_COLOR_INDEX]])
+                self.norm_data.append({'label': data_row['label'],
+                                       'values': norm_data_points,
+                                       'color': data_row['color']})
 
     def calculate_spread(self):
         self.spread = float(self.maximum_value) - float(self.minimum_value)
@@ -594,18 +666,28 @@ class Base(object):
             else:
                 line_number_width = longest_left_label_width + LABEL_MARGIN * 2
 
-            tmp = 0.0 if self.y_axis_label is None else self.marker_caps_height + LABEL_MARGIN * 2
+            if self.y_axis_label is None:
+                tmp = 0.0
+            else:
+                tmp = self.marker_caps_height + LABEL_MARGIN * 2
             self.graph_left = self.left_margin + line_number_width + tmp
 
             # Make space for half the width of the rightmost column label.
-            # Might be greater than the number of columns if between-style bar markers are used.
+            # Might be greater than the number of columns if between-style bar
+            # markers are used.
             tmp = self.labels.keys()
             tmp.sort()
             if len(tmp):
                 last_label = int(tmp[-1])
             else:
                 last_label = 0
-            extra_room_for_long_label = self.calculate_width(self.marker_font_size, self.labels[last_label]) / 2.0 if (last_label >= (self.column_count-1) and self.center_labels_over_point) else 0
+            if last_label >= (self.column_count - 1) and \
+               self.center_labels_over_point:
+                extra_room_for_long_label = \
+                        self.calculate_width(self.marker_font_size,
+                                             self.labels[last_label])
+            else:
+                extra_room_for_long_label = 0
             self.graph_right_margin = self.right_margin + extra_room_for_long_label
 
             self.graph_bottom_margin = self.bottom_margin + self.marker_caps_height + LABEL_MARGIN
@@ -615,11 +697,20 @@ class Base(object):
 
         # When self.hide title, leave a title_margin space for aesthetics.
         # Same with self.hide_legend
-        tmp = self.title_margin if self.hide_title else self.title_caps_height  + self.title_margin
-        tmp += self.legend_margin if self.hide_legend else self.legend_caps_height + self.legend_margin
+        if self.hide_title:
+            tmp = self.title_margin
+        else:
+            tmp = self.title_caps_height + self.title_margin
+        if self.hide_legend:
+            tmp += self.legend_margin
+        else:
+            tmp += self.legend_caps_height + self.legend_margin
         self.graph_top = self.top_margin + tmp
 
-        x_axis_label_height = 0.0 if self.x_axis_label is None else self.marker_caps_height + LABEL_MARGIN
+        if self.x_axis_label is None:
+            x_axis_label_height = 0.0
+        else:
+            x_axis_label_height = self.marker_caps_height + LABEL_MARGIN
         self.graph_bottom = self.raw_rows - self.graph_bottom_margin - x_axis_label_height
         self.graph_height = self.graph_bottom - self.graph_top
 
@@ -630,9 +721,40 @@ class Base(object):
         self.setup_drawing()
 
     def write(self, filename="graph.png"):
+        """draw graph and save the graph image.
+        """
         self.draw()
         self.base_image.write(filename)
 
     def display(self):
+        """wrapper for :func:`pgmagick.Image.display`
+
+        draw graph, and render graph.
+        """
         self.draw()
         self.base_image.display()
+
+
+class StackedMixin(object):
+    """Stacked Graph Mix-in
+
+    When you create new stacked graph object, mixed in this object::
+
+        from gruffy.base import Base, StackedMixin
+
+        class NewStackedGraph(Base, StackedMixin):
+            pass
+    """
+
+    def get_maximum_by_stack(self):
+        """get sum of each stack"""
+        max_hash = {}
+        for data_set in self.gdata:
+            for i, data_point in enumerate(data_set['values']):
+                if i not in max_hash:
+                    max_hash[i] = 0.0
+                max_hash[i] += float(data_point)
+        for key in max_hash.keys():
+            if max_hash[key] > self.maximum_value:
+                self.maximum_value = max_hash[key]
+            self.minimum_value = 0
